# # #
# /5/menu.rb
# (c) 2009 Jan Lelis <mail@janlelis.de>. MIT License.
# See: https://rbjl.janlelis.com/5-playing-with-dijkstra
# # #

# as you might have thought - the program menu
module Menu
  # show the menu
  def self.show(flash=nil)
    system("clear") #print "\e[2J\e[f" - low level function... will not work @ some systems :(
    puts "== AKMRubyLabyrinth #{Version} coded for AKMProgChallenge09 =="
    puts
    puts 'Current configuration'
    puts " LABYRINTH-FILE:\t#{ Config[:file]   || '---'}"
    puts " ALGORITHM:     \t#{(Config[:algo]   || 'Dijkstra [standard]').to_s.capitalize}"
    puts " OUTPUT-FORMAT: \t#{(Config[:output] || 'Console  [standard]').to_s.capitalize}"
    puts
    puts 'Please type one of the commands:'
    puts " info  \t- Give some information"
    puts " file  \t- Read-in the labyrinth-file"
    puts " algo  \t- Select algorithm to use"
    puts " output\t- Select output format to use"
    puts " start \t- Calculate the solution path! [labyrinth file must be loaded]"
    puts " quit  \t- Exit program"
    if flash
      puts
      puts 'Message:'
      puts ' ' << flash
    end
    puts
  end

  # some infos ;)
  def self.info
    system("clear")
    puts '== Info ==
 This program calculates the shortest path in a 9x9 labyrinth. The player
 starts in the top left corner and wants to reach the bottom right one.
 For more info about the task, see <https://akamentor.net/prog/>

 You can choose between following algorithms:
  - Dijkstra
  - Astar
  - Astar_b (variation)
  - Backtracking

 Additionally you can choose between the output formats:
  - Console
  - Html
  - Both
  - Clean (puts out only the solution path)

 The output is a line of commands describing where to go from the start point:
  - u: up
  - d: down
  - l: left
  - r: right

 This program is written in ruby, the shiny great programming language. If you
 do not know it - feel free to study the source of this program and learn it ;)
 If your thoughts the same as mine and you rather like writing nice code than
 super performant, but unreadable c-or-whatever code... than do it!

 Feel free to mail me about the program or ruby: <mail@janlelis.de>

 Some more information about this program can be found in the readme file.

> Press <Enter>'
  end

  # and the main method... idling in the menu till user decides to quit
  def self.idle(lab=nil)
    @lab = lab if lab
    show
    print '>> '; input = gets.strip.downcase.to_sym # get user command
    until input == :quit; begin # extra block for rescueing Err
      case input
        when :file
          temp_file = get_setting "Please enter the path to the labyrinth file\n (base path is: \"#{File.expand_path(Config::BasePathLabs)}\")"
          @lab = Env.new  temp_file # try to load specified file (throws exception if not sucessful)
          Config[:file] = temp_file
          show
        when :algo   then Config[:algo]   = get_setting 'Please enter the algorithm to use';     show
        when :output then Config[:output] = get_setting 'Please enter the output format to use'; show
        when :info   then info;       gets; show
        when :start  then start @lab; gets; show # 'global' method - start the solution!
      end
    rescue ArgumentError => e # error! inform user what he did wrong.
    #  show e.message
    puts '> ' << e.message
    else # no errors
    #  show
    ensure # do this always [errors or no errors]
      print '>> '; input = gets.strip.downcase.to_sym # - get next command
    end; end
  end

  # helper method which gets the settings that we want!
  def self.get_setting(instruction='Please enter a value for this setting') # gets the value of one setting
    print '> ' << instruction << ': '; gets.strip.downcase.to_sym
  end
end
